#include "mainwindow.h"
#include <QFileDialog>
#include <QMessageBox>
#include <QContextMenuEvent>
#include <QMenu>
#include <QMenuBar>
#include <QFile>
#include <QApplication>
#include "canimal.h"
#include <QLabel>
#include <QBoxLayout>
#include <QLineEdit>
using namespace std;

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent){
    saveAct = new QAction(tr("&Save"), this);
    saveAct->setShortcuts(QKeySequence::Save);
    connect(saveAct, &QAction::triggered, this, &MainWindow::save);
    
    newAct = new QAction(tr("&New"), this);
    newAct->setShortcuts(QKeySequence::New);
    connect(newAct, &QAction::triggered, this, &MainWindow::clear);
    
    exitAct = new QAction(tr("E&xit"), this);
    exitAct->setShortcuts(QKeySequence::Quit);
    connect(exitAct, &QAction::triggered, this, &QWidget::close);
    
    aboutAct = new QAction(tr("&About"), this);
    aboutAct->setShortcuts(QKeySequence::HelpContents);
    connect(aboutAct, &QAction::triggered, this, &MainWindow::about);
    
    aboutQtAct = new QAction(tr("About &Qt"), this);
    connect(aboutQtAct, &QAction::triggered, qApp, &QApplication::aboutQt);
    
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(saveAct);
    fileMenu->addAction(newAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);
    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);
    setWindowTitle(tr("oop_lab8_prog"));
    
    // Організація віджетів за допомогою QVBoxLayout
    QVBoxLayout *layout = new QVBoxLayout;
    
    // Організація першої пари полів (Cat)
    QHBoxLayout *catLayout = new QHBoxLayout;
    QLabel *catLabel = new QLabel(tr("Cat:"), this);
    catLayout->addWidget(catLabel);
    catSpinBox = new QDoubleSpinBox(this);
    catSpinBox->setMinimum(0);
    catSpinBox->setMaximum(1000);
    catSpinBox->setValue(5); // Початкове значення
    catSpinBox->setSingleStep(0.1);
    catSpinBox->setDecimals(1);
    catLayout->addWidget(catSpinBox);
    catLineEdit = new QLineEdit(this);
    
    // Організація другої пари полів (Dog)
    QHBoxLayout *dogLayout = new QHBoxLayout;
    QLabel *dogLabel = new QLabel(tr("Dog:"), this);
    dogLayout->addWidget(dogLabel);
    dogSpinBox = new QDoubleSpinBox(this);
    dogSpinBox->setMinimum(0);
    dogSpinBox->setMaximum(1000);
    dogSpinBox->setValue(10);
    dogSpinBox->setSingleStep(0.2);
    dogSpinBox->setDecimals(1);
    dogLayout->addWidget(dogSpinBox);
    dogLineEdit = new QLineEdit(this);
    
    // Організація третьої пари полів (Horse)
    QHBoxLayout *horseLayout = new QHBoxLayout;
    QLabel *horseLabel = new QLabel(tr("Horse:"), this);
    horseLayout->addWidget(horseLabel);
    horseSpinBox = new QDoubleSpinBox(this);
    horseSpinBox->setMinimum(0);
    horseSpinBox->setMaximum(1000);
    horseSpinBox->setValue(400); // Початкове значення
    horseLayout->addWidget(horseSpinBox);
    horseSpinBox->setDecimals(1);
    horseLineEdit = new QLineEdit(this);
    
    QHBoxLayout *umwwLayout = new QHBoxLayout;
    QLabel *umwwLabel = new QLabel(tr("Amogus:"), this);
    umwwLayout->addWidget(umwwLabel);
    umwwSpinBox = new QDoubleSpinBox(this);
    umwwSpinBox->setMinimum(0);
    umwwSpinBox->setMaximum(10000);
    umwwSpinBox->setValue(40); // Початкове значення
    umwwLayout->addWidget(umwwSpinBox);
    umwwSpinBox->setDecimals(1);
    umwwSpinBox->setSingleStep(0.5);
    umwwLineEdit = new QLineEdit(this);
    
    catLineEdit->setText("Frisky");
    dogLineEdit->setText("Bobik");
    horseLineEdit->setText("Juan");
    umwwLineEdit->setText("Olegator");
    
    catLayout->addWidget(catLineEdit);
    layout->addLayout(catLayout);
    dogLayout->addWidget(dogLineEdit);
    layout->addLayout(dogLayout);
    horseLayout->addWidget(horseLineEdit);
    layout->addLayout(horseLayout);
    umwwLayout->addWidget(umwwLineEdit);
    layout->addLayout(umwwLayout);
    
    submitButton = new QPushButton(tr("Submit"), this);
    connect(submitButton, &QPushButton::clicked, this, &MainWindow::handleSubmit);
    layout->addWidget(submitButton);
    
    textEdit = new QTextEdit(this);
    layout->addWidget(textEdit);
    
    QWidget *centralWidget = new QWidget(this);
    centralWidget->setLayout(layout);
    setCentralWidget(centralWidget);
    setMinimumSize(400, 340);
    resize(400,0);
}
void MainWindow::handleSubmit() {
    CAnimal* animals[4];
    
    animals[0] = new CCat(fromLineEdit(catLineEdit), catSpinBox->value());
    animals[1] = new CDog(fromLineEdit(dogLineEdit), dogSpinBox->value());
    animals[2] = new CHorse(fromLineEdit(horseLineEdit), horseSpinBox->value());
    animals[3] = dynamic_cast<CCat*>(new Cumww(fromLineEdit(umwwLineEdit), umwwSpinBox->value()));
    
    for (int i = 0; i < 4; ++i) {
        if (auto Cumww = dynamic_cast<class Cumww*>(animals[i])) {
            textEdit->append(QString::fromUtf8(Cumww->whoiam())+" Calories per day for " + QString::fromUtf8(Cumww->CCat::GetName()) + ": " + QString::number(Cumww->GetFoodCaloriesPerDay(),'f',0));
        } else if (auto cat = dynamic_cast<CCat*>(animals[i])) {
            textEdit->append(QString::fromUtf8(cat->whoiam())+" Calories per day for " + QString::fromUtf8(cat->GetName()) + ": " + QString::number(cat->GetFoodCaloriesPerDay(),'f',0));
        } else if (auto dog = dynamic_cast<CDog*>(animals[i])) {
            textEdit->append(QString::fromUtf8(dog->whoiam())+" Calories per day for " + QString::fromUtf8(dog->GetName()) + ": " + QString::number(dog->GetFoodCaloriesPerDay(),'f',0));
        } else if (auto horse = dynamic_cast<CHorse*>(animals[i])) {
            textEdit->append(QString::fromUtf8(horse->whoiam())+" Calories per day for " + QString::fromUtf8(horse->GetName()) + ": " + QString::number(horse->GetFoodCaloriesPerDay(),'f',0));
        } else textEdit->append("Unknown animal type");
    }
    
    textEdit->append("Total per month: "+QString::number(sum(animals,4)));
    
    for (CAnimal* animal : animals)delete animal;
    submitButton->setText("Append");
}



#ifndef QT_NO_CONTEXTMENU
void MainWindow::contextMenuEvent(QContextMenuEvent *event)
{
    QMenu menu(this);
    
    menu.addAction(saveAct);
    menu.addAction(newAct);
    menu.addAction(aboutAct);
    menu.exec(event->globalPos());
}
#endif // QT_NO_CONTEXTMENU

void MainWindow::save()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Зберігти файл"), "", tr("Текстові файли (*.txt)"));
    if (fileName.isEmpty()) return;
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        QMessageBox::warning(this, tr("Помилка"), tr("Файл неможливо відкрити для запису."));
        return;
    }
    
    QTextStream out(&file);
    out << textEdit->toPlainText();
    if (!textEdit->toPlainText().endsWith("\n"))out << '\n';
    
    file.close();
}

void MainWindow::about()
{
    QMessageBox::about(this, tr("About oop_lab8_prog"),
                       tr("The <b>oop_lab8_prog</b> is a horrible example of laboratory work.<br>Created by Ivan."));
}

void MainWindow::clear()
{
    textEdit->clear();
    submitButton->setText("Submit");
}
